""" ifrit package interfaces

$Id: interfaces.py,v 1.1 2007/02/07 23:27:48 tseaver Exp $
"""
from zope.interface import Interface

class IElement(Interface):
    """ Adapter interface.
    """

class IElementFactory(Interface):
    """ Adapter factory interface.
    """
    def __call__(context):
        """ Adapt context to an element interface
        
        o Adapted object will provide an interface derived from IElement.
        """

class IXMLSerialization(Interface):
    """ Utility interface.
    """
    def serialize(buffer):
        """ Write an XML representation of 'context' into 'buffer'.
        """



import unittest
from StringIO import StringIO

from zope.interface import Interface
from zope.app.testing.placelesssetup import PlacelessSetup
from zope.app.testing import ztapi

class OFSFileAsXMLTests(unittest.TestCase):

    def _getTargetClass(self):
        from ifrit.adapters import OFSFileAsXML
        return OFSFileAsXML

    def _makeOne(self, data):
        file = DummyFile(data)
        return self._getTargetClass()(file)

    def test_context_with_string_data(self):
        from ifrit.interfaces import IElement
        _STRING_PROFILE = """<?xml version="1.0"?><profile/>"""
        adapted = self._makeOne(_STRING_PROFILE)
        buffer = StringIO()
        adapted.serialize(buffer)
        self.assertEqual(buffer.getvalue(), _STRING_PROFILE)

    def test_context_with_unicode_data(self):
        from ifrit.interfaces import IElement
        _UNICODE_PROFILE = u"""<?xml version="1.0"?><profile/>"""
        adapted = self._makeOne(_UNICODE_PROFILE)
        buffer = StringIO()
        adapted.serialize(buffer)
        self.assertEqual(buffer.getvalue(), _UNICODE_PROFILE.encode('utf8'))

    def test_context_with_pdata_data(self):
        from ifrit.interfaces import IElement
        _PDATA_PROFILE = DummyPData('<?xml version="1.0"?>',
                                    DummyPData('<profile/>'))
        adapted = self._makeOne(_PDATA_PROFILE)
        buffer = StringIO()
        adapted.serialize(buffer)
        self.assertEqual(buffer.getvalue(), '<?xml version="1.0"?><profile/>')


class ElementMakerTests(unittest.TestCase, PlacelessSetup):

    def setUp(self):
        PlacelessSetup.setUp(self)

    def tearDown(self):
        PlacelessSetup.tearDown(self)

    def _makeOne(self, path=None, *arg, **kw):
        from zope.interface import directlyProvides
        from ifrit.adapters import ElementMaker
        file = DummyFile()
        directlyProvides(file, IDummy)
        return ElementMaker(path)(file)

    def _fakeSerializer(self, data):
        from ifrit.interfaces import IXMLSerialization
        class DummyAdapter:
            def __init__(self, context):
                pass
            def serialize(self, buffer):
                buffer.write(data)
        ztapi.provideAdapter(IDummy, IXMLSerialization, DummyAdapter, '')

    def test_with_no_path_provides_IElement(self):
        from zope.interface.verify import verifyObject
        from ifrit.interfaces import IElement
        _PROFILE = '<?xml version="1.0"?><profile><education/></profile>'
        self._fakeSerializer(_PROFILE)
        adapted = self._makeOne(path=None)
        verifyObject(IElement, adapted)
        self.assertEqual(adapted.tag, 'profile')
        self.assertEqual(adapted.find('./education').tag, 'education')

    def test_with_path_provides_IElement(self):
        from zope.interface.verify import verifyObject
        from ifrit.interfaces import IElement
        _PROFILE = '<?xml version="1.0"?><profile><education/></profile>'
        self._fakeSerializer(_PROFILE)
        adapted = self._makeOne(path='./education')
        verifyObject(IElement, adapted)
        self.assertEqual(adapted.tag, 'education')

    def test_with_invalid_path_returns_None(self):
        _PROFILE = '<?xml version="1.0"?><profile><education/></profile>'
        self._fakeSerializer(_PROFILE)
        adapted = self._makeOne(path='./nonesuch')
        self.assertEqual(adapted, None)

class IDummy(Interface):
    pass

class DummyFile:
    def __init__(self, data=''):
        self.data = data

class DummyPData:
    def __init__(self, data, next=None):
        self.data = data
        self.next = next

def test_suite():
    from zope.testing.doctest import DocFileTest
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(OFSFileAsXMLTests))
    suite.addTest(unittest.makeSuite(ElementMakerTests))
    suite.addTest(DocFileTest('README.txt', package='ifrit'))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




""" ifrist adapters

$Id: adapters.py,v 1.2 2007/02/09 13:47:31 tseaver Exp $
"""
import warnings

from StringIO import StringIO

from zope.interface import implements
from zope.interface import classImplements
from zope.interface import directlyProvides

from ifrit.interfaces import IElement
from ifrit.interfaces import IElementFactory
from ifrit.interfaces import IXMLSerialization

try:
    from lxml.etree import XML as _XML
    from lxml.etree import ElementBase
    from lxml.etree import ElementDefaultClassLookup
    from lxml.etree import XMLParser

    class _MarkableElement(ElementBase):
        implements(IElement)

    _lookup = ElementDefaultClassLookup(element=_MarkableElement)
    _parser = XMLParser()
    _parser.setElementClassLookup(_lookup)

    def XML(stringvalue):
        return _XML(stringvalue, _parser)

except ImportError:  # fall back to Python's elementtree
    warnings.warn('ifrit: No lxml, falling back pure-python elementtree')
    from elementtree.ElementTree import XML

class OFSFileAsXML(object):
    """ "Convert" OFS.Image.File-like object to XML.

    o Assume that the 'data' element is a string, unicode, or a pdata chunk,
      containing the actual XML to be returned.
    """
    implements(IXMLSerialization)

    def __init__(self, context):
        self.context = context

    def serialize(self, buffer):
        data = self.context.data
        if isinstance(data, str):
            buffer.write(data)
        elif isinstance(data, unicode):
            buffer.write(data.encode('utf8'))
        else:
            while data is not None:
                buffer.write(data.data)
                data = data.next

class ElementMaker(object):
    """ Adapter factories which use paths to find nodes.
    """
    implements(IElementFactory)

    def __init__(self, path=None, marker=IElement):
        self.path = path
        self.marker = marker

    def __call__(self, context):
        buffer = StringIO()

        adapter = IXMLSerialization(context)
        adapter.serialize(buffer)
        tree = XML(buffer.getvalue())

        if self.path is not None:
            node = tree.find(self.path)
        else:
            node = tree

        if node is not None:
            directlyProvides(node, self.marker)

        return node



""" ifrit ZCML directives

$Id: zcml.py,v 1.1 2007/02/08 03:05:18 tseaver Exp $
"""
from zope.component import provideAdapter
from zope.interface import Interface
from zope.schema import TextLine
from zope.configuration.fields import GlobalInterface
from zope.configuration.fields import GlobalObject
from zope.configuration.fields import Path
from zope.configuration.fields import PythonIdentifier
from zope.configuration.fields import Tokens

from ifrit.interfaces import IElement
from ifrit.adapters import ElementMaker

class IPathAdapter(Interface):
    """ Interface for directive which creates a path-based adapter.

    E.g.:

     <ifrit:path_adapter
       for="package.interfaces.IContent"
       name="adapterName"
       path="some_lxml_path"
       marker="package.interfaces.IMyElement"
       module="package.module"
       factory_name="factoryName"
       />

    o Create an adapter factory which uses 'path' to extract a node
      from the serialized XML, and stamps the node with the marker
      interface passed in 'marker'.

    o Bind that factory into 'module' under 'factory_name'.

    o Register that factory as providing an 'IElement' adapter for
      the 'for' interface, using the 'name' if provided (otherwise,
      registers it as the default adapter).
    """
    module = GlobalObject(
        title=u"Target module",
        description=u"Module into which the generated adapter factory "
                    u"will be added.",
        required=True,
        )

    factory_name = PythonIdentifier(
        title=u"Factory name",
        description=u"Name of the generated adapter factory.",
        required=True,
        )

    path = TextLine(
        title=u"Path",
        description=u"XPath or ElementTree path expression",
        required=True,
        )

    for_ = Tokens(
        title=u"Specifications to be adapted",
        description=u"This should be a list of interfaces or classes",
        value_type=GlobalObject(missing_value=object()),
        required=False,
        )

    name = PythonIdentifier(
        title=u"Adapter name",
        description=u"Name under which the adapter is registered.",
        required=False,
        )

    marker = GlobalInterface(
        title=u"Merker Interface",
        description=u"Interface to be stamped onto adapters",
        required=False,
        )


def createPathAdapter(module, factory_name, path,
                      for_=None, name='', marker=None):
    """ Create an adapter factory using 'path'.
 
    o Seat the new schema into 'module' under 'factory_name'.
    """
    if marker is None:
        marker = IElement
    factory = ElementMaker(path, marker)
    setattr(module, factory_name, factory)
    provideAdapter(factory, for_, IElement, name)

def PathAdapterDirective(_context, module, factory_name, path,
                         for_=None, name='', marker=None):
    # Directive handler for <ifrit:path_adapter> directive.
    _context.action(
        discriminator = (module, factory_name,),
        callable = createPathAdapter,
        args = (module, factory_name, path, for_, name, marker),
        )
